
%% Preamble
% This program replicates the main results presented in


% Macroecnomic Factors and Micro-level Bank Behavior
% Claudia Buch, Sandra Eickmeier, Esteban Prieto
% Forthcoming in the Journal of Money, Credit and Banking


% The program prouces as output on the screen tables corresponding to:
% Table 1, Table2, Table 4 and Table 5 in the text.

% The program produces 5 plots corresponding to the figures:
% Figure 1, Figure 2, Figure 3, Figure 4 and Figure 5

% ------------------------------------------------------------------------
% ------------------------------------------------------------------------
% To replicate the regression results, run the Stata do file
% REPLICA_REGRESSIONS_BuchEickmeierPrieto.do
% Note that you first have to run this file before running Stata
% as Stata will need as input data produced in this program
% ------------------------------------------------------------------------
% ------------------------------------------------------------------------


% Disclaimer: Although all efforts have been taken to eliminate errors in
% the data preparation, calculations and coding of estimators, we can not
% give any warranty.

% Last Revision: 10/25/2012

%% loading data, seasonal and transform
clear all
close all
clc
rand('state',56);
randn('seed',3);

remove = 1;
Vars = 5;
p = 1;
CI = 0.90;
rplusM = 6+4;
explvarregr =0;
compare =1;
VarianceDecomposition = 1;

load('MacroData.txt');
OBS = MacroData;

load('BankData.txt');

[MM,WW,Xqoq] = transfstandard(BankData); % standardizing variables
clear MM

Correlation = corrcoef(Xqoq(:,1:5));

disp('                                                                    ');
disp('                                                                    ');
disp('--------------------------------------------------------------------');
disp('--------------------------------------------------------------------');
disp 'Table 1 Correlation btw Banking Variables' ;
disp(round(100*Correlation)/100);
disp('--------------------------------------------------------------------');
disp('                                                                    ');
disp('                                                                    ');

% % --------------------------------------------------------------------

SE = repmat([1;94],1,size(Xqoq,2));
SE(2,:) = SE(2,:) - 1;
clear X t n nn
%% preliminaries: set parameters

mp_position = rplusM;
p = 1;
cr = 0.0005;
Xmes = Xqoq(:,1:5); % save the median bank variables (for later)
Xqoq(:,1:5) = []; % remove the median bank variables
[T,N] = size(Xqoq);
M = size(OBS,2);
r = rplusM - M;

% estimate first rplusM factors from Y
normaliz = 0;
[F0,~,~,~] = factor_estim(Xqoq,rplusM,0); %factor estimation with principal components

[nnn ttt] = size(Xqoq);
nnnTmp = ttt;
nnnTmp = nnnTmp/5;

rsq=[];
for i = 0:4
    [perc_expl, perc_expl_accum] = totalvar(Xqoq(:,1+i*nnnTmp:(i+1)*nnnTmp));
    P(:,i+1) = perc_expl_accum(1:15);
  end

disp('                                                                    ');
disp('                                                                    ');
disp('--------------------------------------------------------------------');
disp('--------------------------------------------------------------------');
disp 'Table 2 Variance explained of each variable' ;
disp(round(100*P)/100);
disp('--------------------------------------------------------------------');
disp('                                                                    ');
disp('                                                                    ');


% get rid of observables from space spanned by rplusM factors according to Boivin-Giannoni

F  = RemObs(OBS,SE,Xqoq,cr,remove,r,rplusM,N,T);

Xqoq = [Xmes Xqoq]; % Add median bank variables again
[T,N] = size(Xqoq);

% Variance explained by observables and factors
XXX = [OBS(:,1) OBS(:,2) OBS(:,3) F(:,1:r) OBS(:,4)];
lam = zeros(rplusM,N); CC= zeros(size(XXX,1),N);IC= zeros(size(XXX,1),N); u = [];
Rsq = [];
for i = 1:N
    res = ols(Xqoq(:,i),XXX);
    Rsq(1,i) = res.rsqr;
end
disp('                                                                    ');
disp('                                                                    ');
disp('--------------------------------------------------------------------');
disp('--------------------------------------------------------------------');
display('Variance Explained by Obseravble and Latent Factors')
disp(ceil(100*mean(Rsq))/100)
disp('--------------------------------------------------------------------');
disp('                                                                    ');
disp('                                                                    ');


% -------------------------------------------------------------------------
clear res;
% -------------------------------------------------------------------------
% calculate loadings, idios. component, common component
for i = 1:N
    res = ols(Xqoq(SE(1,i):SE(2,i),i),XXX(SE(1,i):SE(2,i),:));
    lam(:,i) = res.beta;
    CC(:,i) = XXX*lam(:,i);
    IC(SE(1,i):SE(2,i),i) = Xqoq(SE(1,i):SE(2,i),i)-CC(SE(1,i):SE(2,i),i);
end


% Starting VAR analysis
clear L c cc h i ic nor r x

% construct matrizes
regr = [];
for i = 0:p-1
    regr = [regr XXX(p-i:end-i-1,:)];
end
Y = XXX(p+1:end,:);
X = regr;
clear regr

X = [X ones(length(X),1)]; % X plus a constant

% the ols estimator for the VAR system
B1 = inv(X'*X)*(X'*Y);

% start bootstarp after bootstrap procedure
% remove small sample bias from coeffs
[B, u, u_boot] = BootaBoot_1( B1, X, Y,rplusM,T,p,XXX);
% Generating VCV Matrix Of Errors
sig = cov(u);
% Generating Chol Residuals
D = chol(sig)';
u_orth = u*inv(D)';
D(:,mp_position) = -D(:,mp_position);
%normalize shocks

% Estimate strcutral shocks
s = 20; %the horizon
j = 0;
ttt = 0;
mm = 100;

while j < mm
    ttt = ttt+1;
    % set up Q matrix
    MM = 2;
    clear Q;
    Q = eye(MM);
    newmatrix=normrnd(0,1,MM,MM);
    [Q,R]=qr(newmatrix);
    for i=1:MM;
        if R(i,i)<0
            Q(:,i)=-Q(:,i);
        end
    end
    
    % generate orthogonalized impulses using cholesky decomposition
    % 1. generate the  colesky decomposition
    
    %generating the correct Q matrix
    HHH = eye(rplusM);
    HHH(1:2,1:2) = Q;
    Q = HHH;
    clear HHH;
    QQ = Q';
    A = D*QQ';
    
    % 2. generate the impulse response functions
    
    
    I = eye(rplusM*p,rplusM*p);
    BBB = [B(1:rplusM*p,:)';I(1:end-rplusM,:)]; %companion matrix
    temp= eye(size(BBB));
    irftemp = [];
    irftemp(:,:,1) = temp;
    for h = 1:s % this loop generates the VMA terms
        temp = BBB*temp;
        irftemp(:,:,h+1) = temp;
    end;
    
    
    imp = [];
    imp1 = [];
    for h = 0:s
        BB = irftemp(1:rplusM,1:rplusM,h+1)'; %B^h
        imp(:,:,h+1)  = A'*BB;%*lam';%*WW;  %*v'*WW is *C_hat  %here use the VMA terms to compute the IRFs
    end;
    
    % generating the cummulated impulse responses
    
    jj = [1 2 3 ]; % variables in differences
    for k = 1:3
        kkk = jj(k);
        for jjj = 1:rplusM
            imp1(jjj,kkk,:) = cumsum(imp(jjj,kkk,:));
        end
    end
    
    jj = 4:rplusM; % variables in levels
    for k = 1:(rplusM-3)
        kkk = jj(k);
        for jjj = 1:rplusM
            imp1(jjj,kkk,:) = (imp(jjj,kkk,:));
        end
    end
    
    
    % check restrictions
    % now check the signs of the impulse responses
    % 1. restrictions on the effects of the supply shock
    a1 = [(((imp1(1,1,1:5)) >= 0)) .* (imp1(1,2,1:5) <= 0) ];
    if max(a1) == 0
        continue;   %The restrictions are not satisfied.  Go the beginning to redraw.
    elseif (min(a1)==0)
        continue;    %The restrictions are not satisfied.  Go the beginning to redraw.
    end
    % 2. restrictions on the effects of the demand shock
    a2 = [(((imp1(2,1,1:5)) >= 0)) .* (imp1(2,2,1:5) >= 0) .* (imp1(2,mp_position,1:5) >= 0)];
    if max(a2) == 0
        continue;   %The restrictions are not satisfied.  Go the beginning to redraw.
    elseif (min(a2)==0)
        continue;    %The restrictions are not satisfied.  Go the beginning to redraw.
    end
    j = j+1;
    IMP(:,:,:,j) = imp1; % saves the cummulated IRF
    IMP_nocum(:,:,:,j) = imp; % saves the uncummulated IRF
    QQQ(:,:,j) = QQ;
    clear imp imp1 Q
end


% calcuate the median values of all the impulse responses which passed
for i = 1:rplusM
    for k = 1:rplusM
        for j = 1:s+1
            IMP1(i,k,j) = median(IMP(i,k,j,:));
            IMP_uncum(i,k,j) = median(IMP_nocum(i,k,j,:));
            IMP_std(i,k,j) = std(IMP_nocum(i,k,j,:));
        end
    end
end

nrest = 2;
horrest = 5;
[I pf] = paganfry2(IMP_nocum,IMP_uncum,IMP_std,rplusM,mm,nrest,horrest);

%    [I pf] = paganfry2(IMP_nocum,IMP_uncum,IMP_std,rplusM,mm);
% extract the QQ matrix
Q = QQQ(:,:,I);
MEDIANPLOT = IMP(:,:,:,I);

for h = 0:s
    irf1pointestimate(:,:,h+1) = IMP_nocum(:,:,h+1,I)*lam*WW; %irfs of variables from large dataset to all rplusM shocks
end

u_struct = u*inv(D*Q')';
u_struct(:,rplusM) = -1*u_struct(:,rplusM);


save u_struct_FM u_struct


% plot the impulse responses of the FRY/PAGAN IMPULSE
%the aggregate variables
jj = [1 2 3 mp_position];
k = 0;
M = 4;
z = zeros(s+1,1);
figure
XX = (0:1:s)';
clear h h_u h_l
for i = 1:M  %for the shocks
    ii = jj(i);
    for j = 1:M
        k = k+1;
        jjj = jj(j); % the variables
        subplot(M,M,k)
        for l = 1:s+1 % the horizons
            h(l,1) = MEDIANPLOT(ii,jjj,l);
        end
        hh = [h];
        axis('tight')
        plot(XX,z,'k',XX,hh,'k','LineWidth',2)
        clear h*
    end
end
close all


% Bootstrap


irf_boot = []; u_struct_boot = []; B_boot = []; fe = []; irf0_boot = [];
fetemp = 0; m = 0; mm = 0; nboot = 500;

irf1 = []; irf2 = [];
while mm < nboot
    mm;
    c = ceil(rand(T-p,1)*(T-p));
    u_boot = u(c,:) - repmat(mean(u(c,:)),size(u_boot,1),1);
    Xpast_boot = [];
    for i=p:-1:1
        Xpast_boot=[Xpast_boot XXX(i,:)];%eigentlich Anfangswerte randomisieren
    end
    Xpast_boot = [ Xpast_boot 1 ];
    X_boot = XXX;
    for t = p+1:T
        X_boot(t,:) = Xpast_boot*B + u_boot(t-p,:);
        Xpast_boot = [X_boot(t,:) Xpast_boot(:,1:end-rplusM-1) ones(1,1) ];
    end
    
    z = X_boot;
    clear X_boot; clear Xpast_boot;
    
    zpast = [];
    for kk = 1:p
        zpast = [zpast z(p-kk+1:end-kk,:)];
    end;
    zcurr = z(p+1:end,:);
    zpast = [zpast ones(T-p,1)];
    
    B_boot = inv(zpast'*zpast)*zpast'*zcurr;
    u_boot = zcurr  - zpast*B_boot;
    clear zcurr; clear zpast;
    H = cov(u_boot);
    D = chol(H)';
    D(:,mp_position) = -D(:,mp_position);
    % set up Q matrix
    
    Atilda = D*Q'; %use the rotation matrix which generates impulse responses closest to the "median model"
    u_orth_boot = u_boot * inv(Atilda)';
    
    I = eye(rplusM*p,rplusM*p);
    BBB = [B_boot(1:rplusM*p,:)';I(1:end-rplusM,:)];
    temp = eye(size(BBB));
    
    irftemp = [];
    irftemp(:,:,1) = temp;
    for h = 1:s
        temp = BBB*temp;
        irftemp(:,:,h+1) = temp;
    end;
    clear temp;
    
    
    irf0_boottmp = []; irf1_boottmp = [];
    for h = 0:s
        BB = irftemp(1:rplusM,1:rplusM,h+1)'; %B^h
        irf0_boottmp(:,:,h+1) = Atilda'*BB; %rplusM x rplusM
        irf1_boottmp(:,:,h+1) = irf0_boottmp(:,:,h+1)*lam(:,1:Vars)*WW(1:Vars,1:Vars); %irfs of variables from large dataset to all rplusM shocks
    end
    clear temp;
    
    
    jj = [1 2 3]; % variables in differences
    for k = 1:3
        kkk = jj(k);
        for jjj = 1:rplusM
            imp1(jjj,kkk,:) = cumsum(irf0_boottmp(jjj,kkk,:));
        end
    end
    
    jj = 4:rplusM; % variables in levels
    for k = 1:(rplusM-3)
        kkk = jj(k);
        for jjj = 1:rplusM
            imp1(jjj,kkk,:) = (irf0_boottmp(jjj,kkk,:));
        end
    end
    
    % 1. restrictions on the effects of the supply shock
    a1 = [(((imp1(1,1,1:5)) >= 0)) .* (imp1(1,2,1:5) <= 0) ];
    if max(a1) == 0
        continue;   %The restrictions are not satisfied.  Go the beginning to redraw.
    elseif (min(a1)==0)
        continue;    %The restrictions are not satisfied.  Go the beginning to redraw.
    end
    % 2. restrictions on the effects of the demand shock
    a2 = [(((imp1(2,1,1:5)) >= 0)) .* (imp1(2,2,1:5) >= 0) .* (imp1(2,mp_position,1:5) >= 0)];
    if max(a2) == 0
        continue;   %The restrictions are not satisfied.  Go the beginning to redraw.
    elseif (min(a2)==0)
        continue;    %The restrictions are not satisfied.  Go the beginning to redraw.
    end
    % if everything is good save the draws
    mm = mm+1;
    irf0_boot(:,:,:,mm) = irf0_boottmp;
    irf1_boot(:,:,:,mm) = irf1_boottmp;
    irfcum_boot(:,:,:,mm) = imp1;
    Atilda0(:,:,mm) = Atilda;  %needed for counterfactual simulation
    B_boot0(:,:,mm) = B_boot; %needed for counterfactual simulation
    clear irf0_boottmp irf_boottmp  imp1
end

for i = 1:rplusM
    for k = 1:rplusM
        for j = 1:s+1
            IMP1_cum(i,k,j) = median(irfcum_boot(i,k,j,:)); %median of cumulated impulses
            IMP(i,k,j) = median(irf0_boot(i,k,j,:)); %median of not cumulated impulses
            IMP1_cum_u(i,k,j) = quantile(irfcum_boot(i,k,j,:),CI); %+1 std_deviation of cumulated impulses
            IMP1_cum_l(i,k,j) = quantile(irfcum_boot(i,k,j,:),(1-CI)); %-1 std_deviation of cumulated impulses
        end
    end
end

IMP_med= irf1_boot;
for k = 1:nboot
    for i = 1:rplusM
        for j = 4
            IMP_med(i,j,:,k)= cumsum(IMP_med(i,j,:,k));
        end
    end
end

for i = 1:rplusM
    for k = 1:Vars
        for j = 1:s+1
            IMP1_med(i,k,j) = median(IMP_med(i,k,j,:));
            IMP1_u_med(i,k,j) = quantile(IMP_med(i,k,j,:),CI);
            IMP1_l_med(i,k,j) = quantile(IMP_med(i,k,j,:),(1-CI));
            
        end
    end
end

% plot the impulse responses the aggregate variables
%
if compare == 1
    save IMP1_cum_FullModel IMP1_cum
    save IMP1_cum_FullModel_u IMP1_cum_u
    save IMP1_cum_FullModel_l IMP1_cum_l
end
jj = [1 2 3 mp_position];
M = 4; %# shocks
plotobs2(jj, M, IMP1_cum, IMP1_cum_u, IMP1_cum_l, s)

% plot the impulse responses for the median banks
MEDIAN = [];
if compare == 1
    save IMP1_med_FullModel IMP1_med
    save IMP1_med_FullModel_u IMP1_u_med
    save IMP1_med_FullModel_l IMP1_l_med
end
jj = [1 2 3 mp_position];
M = 4; %# shocks
MM = Vars; %# Variables
plotmedbank2(jj, M, MM, IMP1_med, IMP1_u_med, IMP1_l_med, s) % see function plotirfmedbank

% plot impulse responses for bank level variables



plotindbank2(irf1pointestimate,lam,jj,s)

load RegressionData.mat
load('BankNr.txt');


RevisionMatlabStata2(IRFregr,lamregr,BankNr,Vars)

% save BankIRFBalanced imp_bank

% Dispersion of Common Component and Idiosyncratic Component
disper = [];
[nnn ttt] = size(Xqoq);
nnnTmp = ttt-5;
nnnTmp = nnnTmp/5;
for t = 1:T
    for i = 0:4
        disper(t,1,i+1) = std(IC(t,5 + 1+i*nnnTmp:5+(i+1)*nnnTmp));
        disper(t,2,i+1) = std(CC(t,5 +1+i*nnnTmp:5+(i+1)*nnnTmp));
    end
end


disp('                                                                    ');
disp('--------------------------------------------------------------------');
disp('--------------------------------------------------------------------');
disp('Table 5: Disperion of Idiosyncratic and common component')
disp(round(100*squeeze(mean(disper,1)))./100)
disp('--------------------------------------------------------------------');
disp('                                                                    ');
disp('                                                                    ');



% Decompositon of forecast error variance -
if VarianceDecomposition == 1
    hor = [5,21];
    horizon = s;
    pIC = 1;
    
    % generate a matrix with all shocks in rows, reposnes of observables and
    % median variables in columns
    
    IRF = [irfcum_boot(:,[1:3,rplusM],:,:),IMP_med(:,:,:,:)];
    
    irfic0 = zeros(size(IRF,3),nboot,M+Vars);
    for i = 1:(Vars)
        tmp = irfic(IC(:,i),pIC,nboot,WW(i,i),20); %this here genertes the irf of the idiosyncratic component
        irfic0(1:size(tmp,1),:,M+i) = tmp;
    end
    clear tmp;
    irfic0(:,:,1:3) = cumsum(irfic0(:,:,1:3),1);
    irfic0(:,:,8) = cumsum(irfic0(:,:,8),1);
    
    irf2irfic = [IRF; zeros(1,M+Vars,21,500)];
    
    for m = 1:nboot %check
        irf2irfic(rplusM+1,:,:,m) = squeeze(irfic0(:,m,:))';
    end
    VD = vardecomp(irf2irfic(:,:,1:s+1,:),hor); %do this with non-normalized IRFs
    
    VDmed = squeeze(median(VD(:,hor,:,:),4));
    
    vard = [];
    for i = 1:length(hor)
        vard = [vard  squeeze(VDmed(:,i,[1:3,rplusM])) squeeze(sum(VDmed(:,i,4:rplusM-1),3)) squeeze(VDmed(:,i,rplusM+1))];
    end
    
    
    
    disp('                                                                    ');
    disp('--------------------------------------------------------------------');
    disp('--------------------------------------------------------------------');
    disp('Table 4: Variance Decomposition')
    disp(round(100*vard)./100)
    disp('--------------------------------------------------------------------');
    disp('                                                                    ');
    disp('                                                                    ');
    
    
end
% Estimate the VAR without micro level information
if compare == 1
    
    EstimateVarwoMicro
    % plot shock series and VAR comparison
    plots_misspecification
end
