function [F,CC,IC,Loading] = factor_estim(Y,r,normaliz)

%Inputs:
%   Y: the T x N dataset of stationary variables
%   r: nb of static factors
%   normaliz: normalization of factors (0) or loadings (0)
%Outputs:
%   F: T x r static factors
%   CC, IC: T x N common and idios. components
%   Loading: N x r factor loadings


[T,N] = size(Y);

if normaliz == 1, %normalization of loadings
    
    [v,d] = eigs(cov(Y),r);
    
    %flip the eigenvectors to preserve signs
    for i=1:r
         if v(1,i) < 0, v(:,i) = -v(:,i); end
    end
    if d(1,1) < d(r,r)
        d = diag(sort(diag(d*(-1)))*(-1));
        v = flipud(v')';
    end    
   
    F = Y*v;   %'d' indicates first diff.            
    Loading = v;
    CC = F*v';
   
elseif normaliz == 0, %normalization of factors (wie BN 2004 im PANIC-Papier)    
    
    [v,d] = eigs(Y*Y',r);
    
    %flip the eigenvectors to preserve signs
    for i=1:r
         if v(1,i) < 0, v(:,i) = -v(:,i); end
    end
    if d(1,1) < d(r,r)
        d = diag(sort(diag(d*(-1)))*(-1));
        v = flipud(v')';
    end    
   
    F = sqrt(T)*v;  
    Loading = Y'*F/T;
    CC = F*Loading';
    
end

IC = Y - CC;

%identical to BN 2004 estimation
%    [v,d] = eigs(Y*Y'/N/T,r);

%    for i=1:r
%        if v(1,i)<0, v(:,i)=-v(:,i); end
%    end
%    if d(1,1)<d(r,r)
%        d=diag(sort(diag(d*(-1)))*(-1));
%        v=flipud(v')';
%    end    

%    F = v*sqrt(T);
%    Loading=Y'*F/T;
        
%    CC = F*Loading';     
%    IC = Y-CC;          
